@@ -1,17 +1,21 @@
 #
 # Indicators to show overbought or oversold position
 # ----------------------------------------------------
+from functools import partial
 import numpy as np
 
 from .base import (
     COMMANDS,
     CommandPreset,
+    ReturnType,
 
     arg_period
 )
 
 from stock_pandas.common import (
-    period_to_int
+    rolling_calc,
+    period_to_int,
+    column_enums
 )
 
 from stock_pandas.math.ma import (
@@ -19,26 +23,66 @@ from stock_pandas.math.ma import (
 )
 
 
+# llv & hhv
+# ----------------------------------------------------
+
+def llv(df, s, period, column) -> ReturnType:
+    """Gets LLV (Lowest of Low Value)
+    """
+
+    return rolling_calc(
+        df[column][s].to_numpy(),
+        period,
+        min
+    ), period
+
+
+COMMANDS['llv'] = (
+    CommandPreset(
+        llv,
+        [
+            arg_period,
+            ('low', column_enums)
+        ]
+    ),
+    None,
+    None
+)
+
+
+def hhv(df, s, period, column) -> ReturnType:
+    """Gets HHV (Highest of High Value)
+    """
+
+    return rolling_calc(
+        df[column][s].to_numpy(),
+        period,
+        max
+    ), period
+
+
+COMMANDS['hhv'] = (
+    CommandPreset(
+        hhv,
+        [
+            arg_period,
+            ('high', column_enums)
+        ]
+    ),
+    None,
+    None
+)
+
+
 # rsv
 # ----------------------------------------------------
 
-def rsv(df, s, period):
+def rsv(column_low, column_high, df, s, period) -> ReturnType:
     """Gets RSV (Raw Stochastic Value)
     """
 
-    # Lowest Low Value
-    llv = df['low'][s].rolling(
-        min_periods=period,
-        window=period,
-        center=False
-    ).min()
-
-    # Highest High Value
-    hhv = df['high'][s].rolling(
-        min_periods=period,
-        window=period,
-        center=False
-    ).max()
+    llv = df.exec(f'llv:{period},{column_low}')[s]
+    hhv = df.exec(f'hhv:{period},{column_high}')[s]
 
     v = (
         (df['close'][s] - llv) / (hhv - llv)
@@ -47,9 +91,13 @@ def rsv(df, s, period):
     return v.to_numpy(), period
 
 
-COMMANDS['rsv'] = CommandPreset(
-    rsv,
-    [arg_period]
+COMMANDS['rsv'] = (
+    CommandPreset(
+        partial(rsv, 'low', 'high'),
+        [arg_period]
+    ),
+    None,
+    None
 )
 
 
@@ -84,7 +132,7 @@ def ewma(
         yield k
 
 
-def kdj_k(df, s, period_rsv, period_k, init):
+def kdj_k(df, s, period_rsv, period_k, init) -> ReturnType:
     """Gets KDJ K
     """
 
@@ -93,25 +141,25 @@ def kdj_k(df, s, period_rsv, period_k, init):
     return np.fromiter(ewma(rsv, period_k, init), float), period_rsv
 
 
-def kdj_d(df, s, period_rsv, period_k, period_d, init):
+def kdj_d(df, s, period_rsv, period_k, period_d, init) -> ReturnType:
     k = df.exec(f'kdj.k:{period_rsv},{period_k},{init}')[s]
 
     return np.fromiter(ewma(k, period_d, init), float), period_rsv
 
 
-def kdj_j(df, s, period_rsv, period_k, period_d, init):
+def kdj_j(df, s, period_rsv, period_k, period_d, init) -> ReturnType:
     k = df.exec(f'kdj.k:{period_rsv},{period_k},{init}')[s]
     d = df.exec(f'kdj.d:{period_rsv},{period_k},{period_d},{init}')[s]
 
     return KDJ_WEIGHT_K * k - KDJ_WEIGHT_D * d, period_rsv
 
 
-def init_to_float(value):
+def init_to_float(raw_value: str) -> float:
     try:
-        value = float(value)
+        value = float(raw_value)
     except Exception:
         raise ValueError(
-            f'init_value must be a float, but got `{value}`'
+            f'init_value must be a float, but got `{raw_value}`'
         )
 
     if value < 0. or value > 100.:
@@ -142,10 +190,11 @@ args_dj = [
 ]
 
 
-COMMANDS['kdj'] = CommandPreset(
+COMMANDS['kdj'] = (
+    None,
     # We must specify sub command for kdj, such as
     # 'kdj.k'
-    subs_map=dict(
+    dict(
         k=CommandPreset(
             kdj_k,
             args_k
@@ -160,14 +209,15 @@ COMMANDS['kdj'] = CommandPreset(
             kdj_j,
             args_dj
         )
-    )
+    ),
+    None
 )
 
 
 # rsi
 # ----------------------------------------------------
 
-def rsi(df, s, period):
+def rsi(df, s, period) -> ReturnType:
     """Calculates N-period RSI (Relative Strength Index)
 
     https://en.wikipedia.org/wiki/Relative_strength_index
@@ -186,7 +236,11 @@ def rsi(df, s, period):
     return 100 - 100 / (1. + smma_u / smma_d), period
 
 
-COMMANDS['rsi'] = CommandPreset(
-    rsi,
-    [arg_period]
+COMMANDS['rsi'] = (
+    CommandPreset(
+        rsi,
+        [arg_period]
+    ),
+    None,
+    None
 )
